
/*
The communication between the basic stamp and the AVR
is between 8 wires (PORTA).The two highest bits are
for the communication, and the lowest 6 bits are for
data.
Pin8 is written by the AVR, and
Pin7 lowest bit is written by the basic stamp.

The following is an example of the communication
protocol (one byte forward, one byte back):

pin8 pin7 data
 0    0         Idle
 0    0    x    AVR puts data on lines
 1    0    x    AVR signals that data is on lines
 1    1    x    PS signals that data is picked up
 1    1         AVR resets direction of pins to 'IN'
 0    1         AVR confirms data is recvd.
 0    0         PS ack that AVR has reset pin direction
 1    0         AVR is ready to recieve data
 1    0         PS changes data direction to 'OUT'
 1    0    x    PS puts data on lines
 1    1    x    PS says that data is on lines
 1    1    x    AVR gets data
 0    1    x    AVR says data is recieved
 0    1         PS change direction of pins to 'IN'
 0    0         PS ack that communication is done
 0    0         AVR changes data direction to 'OUT'
*/

//settings the port directions, 0 is in, 1 is out
//PORTA is for drive motor.
//PORTB is for steering
//write to PORT
//read from PIN
//set direction to DDR

#include <avr/io.h>

#include "BasicStamp2Library.h"



//Define the commands to send
#define MOTOR_FORWARD_SPEED_MAX      0
#define MOTOR_FORWARD_SPEED_MEDIUM   1
#define MOTOR_FORWARD_SPEED_SLOW     2
#define MOTOR_BACKWARD_SPEED_MAX     3
#define MOTOR_BACKWARD_SPEED_MEDIUM  4
#define MOTOR_BACKWARD_SPEED_SLOW    5
#define MOTOR_STOP                   6
#define MOTOR_ENCODER_COUNT          7
#define MOTOR_DONE_MOVING            8
#define SET_LEDS                     9
#define GET_BUTTONS_AND_SWITCHES     10
#define TURN                         11
#define BEEP_SPEAKER                 12
#define SLEEP                        13
//#define POWERSAVE_SLEEP              14
//#define RANDOM                       15

#define BITS_TO_SEND                 12
#define BITS_TO_RECV                 8

/**
 * Initializes PORTA to use the BasicStamp2.
 */
void initBasicStamp()
{
	
	//initialize portA to all in.
	//If the BasicStamp puts something
	//on the lines when it should not,
	//no problems occur.
	
	DDRA = 0xBF;
    PORTA = 0;



    long x=0;
    for(x=0;x < 100000; x++);

}


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void forwardFast(byte ticks)
{
	send(MOTOR_FORWARD_SPEED_MAX, ticks);
}

/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void forwardMedium(byte ticks)
{
    send(MOTOR_FORWARD_SPEED_MEDIUM, ticks);
}

/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
* @param ticks - Ticks to go forward.
 */
void forwardSlow(byte ticks)
{
    send(MOTOR_FORWARD_SPEED_SLOW, ticks);
}


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void backwardFast(byte ticks)
{
	send(MOTOR_BACKWARD_SPEED_MAX, ticks);
}


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void backwardMeduim(byte ticks)
{
    send(MOTOR_BACKWARD_SPEED_MEDIUM, ticks);
}


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void backwardSlow(byte ticks)
{
	send(MOTOR_BACKWARD_SPEED_SLOW, ticks);
}


/**
 * If the motor was requested to move so many
 * ticks, this replys back if those ticks have
 * been moved or not.
 *
 * @return byte - 1 if motor is done, 0 otherwise
 */
byte isMotorDone()
{
    send(MOTOR_DONE_MOVING, 0);
    return recv();
}


/**
 * Stop the motor if it is moving. Return
 * the distance it went since the last 
 * mov't command.
 *
 * @return byte - Distance motor went
 *     since last mov't command.
 */
byte stopMotor()
{
    send(MOTOR_STOP, 0);
    return recv();
}


/**
 * If the motor was requested to move so many
 * ticks, this replys back how many of those
 * ticks have been moved.
 *
 * @return byte - number of ticks
 */
byte getTicks()
{
    send(MOTOR_ENCODER_COUNT, 0);
    return recv();
}


/**
 * Turns the wheels so many degrees to the right of center.
 * Ex, if want to turn toward 50 degrees on circle,
 * input 40 degrees (as 50 degrees is 40 degrees from the
 * 90 degree point).
 *
 * @param degreesFromCenter
 */
void turnRight(byte degreesFromCenter)
{
    byte newDegree;
	if(degreesFromCenter > 60)
	{
	    newDegree = 0x01;
	}
	else
	{
	    newDegree = 0x80 - (degreesFromCenter * 2.11);
	}

	send(TURN, newDegree);
}

/**
 * Turns the wheels so many degrees to the left of center.
 * Ex, if want to turn toward 100 degrees on circle,
 * input 10 degrees (as 100 degrees is 10 degrees from the
 * 90 degree point).
 *
 * @param degreesFromCenter
 */
void turnLeft(byte degreesFromCenter)
{
	//there are 0x01 - 0xFF possible angles.
	//120 degrees possible movement.
	//so 60 degrees to the left possible.
	//0x80 is the middle, leaving 127 ticks
	//to move to left.
	//127 ticks / 60 degrees = 2.11 ticks per degree.
	
    byte newDegree;

	if(degreesFromCenter > 60)
	{
		newDegree = 0xFF;
	}
	else
	{
	    newDegree = 0x80 + (degreesFromCenter * 2.11);
	}
	
	send(TURN, newDegree);
}

/**
 * Sets the wheels to 90 degrees, or straight ahead.
 */
void setStraight()
{
	send(TURN,0x80);
}


/**
 * Turns off the power to the servos
 */
void turnOffSteering()
{
    send(TURN,0x00);
}

/*
 * Sets LEDs to a certain value.
 * 1 means turn on the LED,
 * 0 means turn off the LED
 *
 * @param redLed
 * @param greenLed
 */
void setLeds(byte redLed, byte greenLed)
{
    //The leds are inverted logic.
    //However, send non-inverted logic 
	byte data = ( (greenLed & 0x01) << 1 ) + (redLed & 0x01);
	send(SET_LEDS, data);
}


/*
 * Read in the value of button1
 *
 * @return byte - value of button1
 */
byte isButton1Pushed()
{
	send(GET_BUTTONS_AND_SWITCHES, 0);

	//bit0 = button1
	//bit1 = button2
	//bit2 = switch1
	//bit3 = switch2
	
	return (recv() & 0x01) >> 0;
}

/*
 * Read in the value of button2
 *
 * @return byte - value of button2
 */
byte isButton2Pushed()
{
	send(GET_BUTTONS_AND_SWITCHES, 0);

	//bit0 = button1
	//bit1 = button2
	//bit2 = switch1
	//bit3 = switch2

	return (recv() & 0x02) >> 1;
}

/*
 * Read in the value of jumper1
 *
 * @return byte - value of jumper1
 */
byte isJumper1On()
{
	send(GET_BUTTONS_AND_SWITCHES, 0);

	//bit0 = button1
	//bit1 = button2
	//bit2 = switch1
	//bit3 = switch2

	return (recv() & 0x04) >> 2;
}

/*
 * Read in the value of jumper2
 *
 * @return byte - value of jumper2
 */
byte isJumper2On()
{
	send(GET_BUTTONS_AND_SWITCHES, 0);

	//bit0 = button1
	//bit1 = button2
	//bit2 = switch1
	//bit3 = switch2

	return (recv() & 0x08) >> 3;
}


/*
 * Outputs a beep to the speaker of a certain
 * frequency and a certain duration.
 * Frequency range: 0-1500 hz  (input 0-15)
 * Druation range:  0-1500 ms. (input 0-15)
 *
 * @param frequency - how many 100's of hz.
 *     Ex 500 Hz means frequency=5.
 * @param duration - how many 100's of ms
 *     Ex 1500 ms means duration = 15.
 */
void beep(byte frequency, byte duration)
{
	byte data = ( (frequency & 0x0F) << 4)
	        + (duration & 0x0F);
	send(BEEP_SPEAKER, data);
}

/*
 * Sleep for a number of ms. There is no
 * limit to the length of delay.
 * Note, if a sleep of 0 is inputted,
 * it will result in a delay of about
 * 2 ms.
 * @param delay - how many ms.
 *     Ex 500 ms means delay=500.
 */
void sleep(int delay)
{
	delay = delay / 10;
    while(delay > 255)
    {
        send(SLEEP, 255);
        recv();
        delay = delay - 255;
    }

    send(SLEEP,(byte)delay);
    recv();
}


/**
 * Sends a command and data to the BasicStamp.
 *
 * @param command
 * @param data
 */
void send(byte command, byte data)
{
   //send order:
   //c4 c3
   //c2 c1
   //d8 d7
   //d6 d5
   //d4 d3
   //d2 d1
   //=> toSend = 0  0  0  0  d2 d1 d4 d3 d6 d5 d8 d7 c2 c1 c4 c3
   //   data   =                         d8 d7 d6 d5 d4 d3 d2 d1
   //   command=                                     c4 c3 c2 c1

    int toSend = 0;
    toSend += (command & 0x08) >> 2;
    toSend += (command & 0x04) >> 2;
    toSend += (command & 0x02) << 2;
    toSend += (command & 0x01) << 2;
    toSend += (data    & 0x80) >> 2;
    toSend += (data    & 0x40) >> 2;
    toSend += (data    & 0x20) << 2;
    toSend += (data    & 0x10) << 2;
    toSend += (data    & 0x08) << 6;
    toSend += (data    & 0x04) << 6;
    toSend += (data    & 0x02) << 10;
    toSend += (data    & 0x01) << 10;

    //set data direction to send
   DDRA = 0xBF;
   
   byte sentBits = 0;

   while(sentBits < BITS_TO_SEND)
   {
    
       //put data on port
       PORTA = ( toSend & 0x0003) << 4;

       //mark data ok to read
       PORTA = PINA + 0x80;

       //wait for response
       while( (PINA & 0x40) != 0x40);
   
       //PS has picked up data
       PORTA = 0;
   
       while( (PINA & 0x40) != 0);
   
       //sent
   
       toSend >>= 2;
       sentBits += 2;
   }

   
}

/**
 * Recieves one byte of data from the BasicStamp
 *
 * @return data recieved
 */
byte recv()
{

    //Recv order:
    //d2 d1
    //d4 d3
    //d6 d5
    //d8 d7

    //recv states:
    // 0 0 0 0 0 0 0 0 
    // 0 0 0 0 0 0 2 1 
    // 0 0 0 0 4 3 2 1 
    // 0 0 6 5 4 3 2 1
    // 8 7 6 5 4 3 2 1 

	byte toReturn = 0;
    byte bitsRecvd = 0;
	
	//set data direction
	DDRA = 0x80;

    byte bitsToShift = 6;

    while(bitsRecvd <BITS_TO_RECV)
    {

	    PORTA = 0x80;
	
	    while( (PINA & 0x40) != 0x40);
	
	    //data is on pins
	    toReturn += ((PINA & 0x30) << 2) >> bitsToShift;
        bitsToShift -= 2;

	    PORTA = 0;
	
	    while( (PINA & 0x40) != 0);
	
        bitsRecvd += 2;
    }

	return toReturn;
}
	
	

