
/*
The communication between the basic stamp and the AVR
is between 8 wires (PORTA).The two highest bits are 
for the communication, and the lowest 6 bits are for 
data.
Pin8 is written by the AVR, and
Pin7 lowest bit is written by the basic stamp.

The following is an example of the communication
protocol (one byte forward, one byte back):

pin8 pin7 data
 0    0         Idle
 0    0    x    AVR puts data on lines
 1    0    x    AVR signals that data is on lines
 1    1    x    PS signals that data is picked up
 1    1         AVR resets direction of pins to 'IN'
 0    1         AVR confirms data is recvd.
 0    0         PS ack that AVR has reset pin direction
 1    0         AVR is ready to recieve data
 1    0         PS changes data direction to 'OUT'
 1    0    x    PS puts data on lines
 1    1    x    PS says that data is on lines
 1    1    x    AVR gets data
 0    1    x    AVR says data is recieved
 0    1         PS change direction of pins to 'IN'
 0    0         PS ack that communication is done
 0    0         AVR changes data direction to 'OUT'
*/

//settings the port directions, 0 is in, 1 is out
//PORTA is for drive motor.
//PORTB is for steering
//write to PORT
//read from PIN
//set direction to DDR

#include <avr/io.h>

#define byte char
		
/**
 * Initializes PORTA to use the BasicStamp2.
 */
void initBasicStamp();


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void forwardFast(byte ticks);

/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void forwardMedium(byte ticks);

/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
* @param ticks - Ticks to go forward.
 */
void forwardSlow(byte ticks);


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void backwardFast(byte ticks);


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void backwardMeduim(byte ticks);


/**
 * Sends command to robot to move so many ticks.
 * Command returning is not a guarantee that
 * distance has been traveled.
 * Range is 0-255 ticks. 1 inch is 2.3 ticks.
 *
 * @param ticks - Ticks to go forward.
 */
void backwardSlow(byte ticks);


/**
 * If the motor was requested to move so many
 * ticks, this replys back if those ticks have
 * been moved or not.
 *
 * @return byte - 1 if motor is done, 0 otherwise
 */
byte isMotorDone();


/**
 * Stop the motor if it is moving. Return
 * the distance it went since the last 
 * mov't command.
 *
 * @return byte - Distance motor went
 *     since last mov't command.
 */
byte stopMotor();

/**
 * If the motor was requested to move so many
 * ticks, this replys back how many of those
 * ticks have been moved.
 *
 * @return byte - number of ticks
 */
byte getTicks();


/**
 * Turns the wheels so many degrees to the right of center.
 * Ex, if want to turn toward 50 degrees on circle,
 * input 40 degrees (as 50 degrees is 40 degrees from the
 * 90 degree point).
 *
 * @param degreesFromCenter
 */
void turnRight(byte degreesFromCenter);

/**
 * Turns the wheels so many degrees to the left of center.
 * Ex, if want to turn toward 100 degrees on circle,
 * input 10 degrees (as 100 degrees is 10 degrees from the
 * 90 degree point).
 *
 * @param degreesFromCenter
 */
void turnLeft(byte degreesFromCenter);

/**
 * Sets the wheels to 90 degrees, or straight ahead.
 */
void setStraight();


/**
 * Turns off the power to the servos
 */
void turnOffSteering();

/*
 * Sets LEDs to a certain value.
 *
 * @param byte redLed
 * @param byte greenLed
 */
void setLeds(byte redLed, byte greenLed);


/*
 * Read in the value of button1
 *
 * @return byte - value of button1
 */
byte isButton1Pushed();

/*
 * Read in the value of button2
 *
 * @return byte - value of button2
 */
byte isButton2Pushed();

/*
 * Read in the value of jumper1
 *
 * @return byte - value of jumper1
 */
byte isJumper1On();

/*
 * Read in the value of jumper2
 *
 * @return byte - value of jumper2
 */
byte isJumper2On();


/*
 * Outputs a beep to the speaker of a certain
 * frequency and a certain duration.
 * Frequency range: 0-1500 hz  (input 0-15)
 * Druation range:  0-1500 ms. (input 0-15)
 *
 * @param frequency - how many 100's of hz.
 *     Ex 500 Hz means frequency=5.
 * @param duration - how many 100's of ms
 *     Ex 1500 ms means duration = 15.
 */
void beep(byte frequency, byte duration);


/*
 * Sleep for a number of ms. There is no
 * limit to the length of delay.
 * Note: if the argument is 0, this will
 * result in a sleep of around 2ms.
 * @param delay - how many ms.
 *     Ex 500 ms means delay=500.
 */
void sleep(int delay);

/**
 * Sends a command and data to the BasicStamp.
 *
 * @param command
 * @param data
 */
void send(byte command, byte data);

/**
 * Recieves one byte of data from the BasicStamp
 *
 * @return data recieved
 */
byte recv();
		
